package rwth.i2.ltlrv.afastate.base;

import rwth.i2.ltlrv.formula.interfaze.IFormula;

/**
 * Represents a binary formula, which is commutative. This means, that
 * a formula <i>phi(a,b)</i> is always equal to <i>phi(b,a)</i>. Also
 * their hashcodes match.
 *  
 * @author Eric Bodden
 */
public abstract class CommutativeBinaryAFAState extends BinaryAFAState {

    /**
     * @param subformula1
     * @param subformula2
     */
    public CommutativeBinaryAFAState(IFormula subformula1, IFormula subformula2) {
        super(subformula1, subformula2);
    }
    
    /**
     * Implements an equality check based on the assumption that this formula constructor
     * is commutative.
     * @see Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object oth) {
        if(super.equals(oth)) {
            return true;
        }
        
        if(!(oth instanceof CommutativeBinaryAFAState)) {
            return false;
        }
        
        CommutativeBinaryAFAState other = (CommutativeBinaryAFAState)oth;        
        return (other.getSubformula1().equals(subformula1)
            && other.getSubformula2().equals(subformula2)) ||
               (other.getSubformula1().equals(subformula2)
            && other.getSubformula2().equals(subformula1));
    }
    
    /**
     * Calculates a hash code based on the assumption that this formula constructor
     * is commutative.
     * @see Object#hashCode()
     */
    @Override
    public int hashCode() {
        return subformula1.hashCode() + subformula2.hashCode();
    }
}
