package rwth.i2.ltlrv.afastate.impl;

import java.util.HashSet;
import java.util.Set;

import rwth.i2.ltlrv.afastate.base.CommutativeBinaryAFAState;
import rwth.i2.ltlrv.afastate.interfaze.IAFAState;
import rwth.i2.ltlrv.afastate.interfaze.IAnd;
import rwth.i2.ltlrv.data.PropositionSet;
import rwth.i2.ltlrv.data.WeakValuesMap;
import rwth.i2.ltlrv.formula.interfaze.IFormula;
import rwth.i2.ltlrv.util.SetUtils;

/**
 * And - Implements a conjunct.
 * 
 * @author Eric Bodden
 */
public class And extends CommutativeBinaryAFAState implements IAnd {

    /**
     * Constructs a conjunt of the two subformulae.
     * @param subFormula1 left subformula
     * @param subFormula2 right subformula
     */
    public And(IFormula subFormula1, IFormula subFormula2) {
        super(subFormula1,subFormula2);
    }

    /**
     * {@inheritDoc}
     */
    public boolean isFinalStateInAFA() {
        //only subformulae implementing IAFAState allowed at this stage
        assert (this.subformula1 instanceof IAFAState) && (this.subformula2 instanceof IAFAState);
        //this state is final if both subformulae are final
        return ((IAFAState)this.subformula1).isFinalStateInAFA()
            && ((IAFAState)this.subformula2).isFinalStateInAFA(); 
    }

    /**
	 * {@inheritDoc}
	 */
	@Override
    public void doTransition(Set<String> context, PropositionSet currentProps, WeakValuesMap<String, Object> currentBinding, Set<Set<IAFAState>> result) {
        //only subformulae implementing IAFAState allowed at this stage
        assert (this.subformula1 instanceof IAFAState) && (this.subformula2 instanceof IAFAState);
        //recurse
        Set<Set<IAFAState>> res1 = new HashSet<Set<IAFAState>>();
        ((IAFAState)subformula1).transition(context,currentProps,currentBinding, res1);
        Set<Set<IAFAState>> res2 = new HashSet<Set<IAFAState>>();
        ((IAFAState)subformula2).transition(context,currentProps,currentBinding, res2);
        //conjoin the two results
        SetUtils.clausesetProduct(res1, res2, result);
	}

    /**
     * {@inheritDoc}
     */
    public void providesPos(Set<String> result) {
        //only subformulae implementing IAFAState allowed at this stage
        assert (this.subformula1 instanceof IAFAState) && (this.subformula2 instanceof IAFAState);
        //join results for both subformulae
        ((IAFAState)subformula1).providesPos(result);
        ((IAFAState)subformula2).providesPos(result);
	}

    /**
     * {@inheritDoc}
     */
    public void providesNeg(Set<String> result) {
        //only subformulae implementing IAFAState allowed at this stage
        assert (this.subformula1 instanceof IAFAState) && (this.subformula2 instanceof IAFAState);
        //recurse
        Set<String> res1 = new HashSet<String>();
        ((IAFAState)subformula1).providesNeg(res1);
        Set<String> res2 = new HashSet<String>();
        ((IAFAState)subformula2).providesNeg(res2);
        //intersect both results, cause a binding is certainly provided if it is provided
        //in both branches of the alternative
        for (String varName : res2) {
            if(res1.contains(varName)) {
                result.add(varName);
            }
        }        
    }

    /**
     * {@inheritDoc}
     */
    public IAFAState reconstruct(IFormula t1, IFormula t2) {
        return factory.And(t1, t2);
    }
 
    /**
     * {@inheritDoc}
     */
    public String symbol() {
        return "&&";
    }
    
}
