package rwth.i2.ltlrv.afastate.interfaze;


import rwth.i2.ltlrv.data.WeakValuesMap;

/**
 * IProposition represents a proposition
 * 
 * @author Eric Bodden
 */
public interface IProposition extends INullaryAFAState {
    
    /**
	 * Constant meaning "unbound binding". 
	 */
	public static final Object UNBOUND = new Object() {
	    @Override
        public String toString() {
	        return "UNBOUND";
	    }
	};


	/**
     * Clients should implement this method, since the
     * implementation uses HashSets, which require a proper
     * notion of equality.
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj);
    
    /**
     * Clients should implement this method, since the
     * implementation uses HashSets, which require a proper
     * hash code.
     * @see java.lang.Object#hashCode()
     */
    public int hashCode();
    
    /**
     * Returns <code>true</code> if this proposition matches the
     * given <i>specializedProposition</i>. This is the case when
     * the labels are equal and <i>specializedProposition</i> is
     * at least as special as <code>this</code>, meaning it holds
     * the same formals and at least the formals bound in <code>this</code>
     * are bound in <i>specializedProposition</i>. 
     * @param specializedProposition a specialized proposition
     * @return <code>true</code> if this proposition matches the
     * given <i>specializedProposition</i>
     */
    public boolean matches(IProposition specializedProposition);
    
    /**
     * Returns a copy of all bindings in this proposition. This is a weak map:
     * Values which are mapped may be claimed by the garbage collector. In this case,
     * the mapping will disappear.
     * @return a copy of all bindings in this proposition
     */
    public WeakValuesMap<String,Object> getBindings();
    
    
    /**
     * Specialized return type, for convenience.
     * @see IAFAState#specializeBindings(WeakValuesMap)
     */
    public IProposition specializeBindings(WeakValuesMap<String, Object> bindings);

}
