package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;


/**
 * EvalIfTestEquality
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalIfTestUnequalityConcurrent extends EvalAbstractIfClosureTests {

    private final static IIfClosure[] closureAUnEqB = new IIfClosure[] {
        new IIfClosure() {

            public boolean satisfiedUnderBindings(WeakValuesMap<String, Object> currentBinding) throws UserCausedException {
                Object a = currentBinding.get("a");
                Object b = currentBinding.get("b");
                return a!=b;
            }
            
            public String toString() { return "a!=b"; }

			public String[] variableNames() {
				return new String[] {"a","b"};
			}
            
        }            
    };
    
    private final static IProposition PROP_P_A = f.Proposition("p", new String[] {"a"});
    private final static IProposition PROP_Q_B_BueqA = f.Proposition("q", new String[] {"b"}, closureAUnEqB );

    public EvalIfTestUnequalityConcurrent() {
        super( f.And(PROP_P_A, PROP_Q_B_BueqA ) ); 
    }
    
    public void testPositive() {
        assertFalse(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(PROP_P_A.specializeBindings(bindings));
        
        bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("b","o1");
        props.add(PROP_Q_B_BueqA.specializeBindings(bindings));
        
        config.transition(props);
        assertFalse(config.isFinal());
    }
    
    public void testNegativeWrongArgument() {
        assertFalse(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(PROP_P_A.specializeBindings(bindings));
        
        bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("b","o2");
        props.add(PROP_Q_B_BueqA.specializeBindings(bindings));
        
        config.transition(props);
        assertTrue(config.isFinal());
    }
         
}
